@@ -19,7 +19,6 @@ from sphinx.deprecation import DeprecatedDict, RemovedInSphinx30Warning
 from sphinx.domains import Domain
 from sphinx.locale import _
 from sphinx.util.docutils import SphinxDirective
-from sphinx.util.nodes import set_source_info
 
 
 if False:
@@ -68,7 +67,7 @@ class VersionChange(SphinxDirective):
         # type: () -> List[nodes.Node]
         node = addnodes.versionmodified()
         node.document = self.state.document
-        set_source_info(self, node)
+        self.set_source_info(node)
         node['type'] = self.name
         node['version'] = self.arguments[0]
         text = versionlabels[self.name] % self.arguments[0]
@@ -76,7 +75,7 @@ class VersionChange(SphinxDirective):
             inodes, messages = self.state.inline_text(self.arguments[1],
                                                       self.lineno + 1)
             para = nodes.paragraph(self.arguments[1], '', *inodes, translatable=False)
-            set_source_info(self, para)
+            self.set_source_info(para)
             node.append(para)
         else:
             messages = []
