@@ -1,31 +1,29 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.domains.javascript
     ~~~~~~~~~~~~~~~~~~~~~~~~~
 
     The JavaScript domain.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from typing import TYPE_CHECKING
-
 from docutils import nodes
-from docutils.parsers.rst import Directive, directives
+from docutils.parsers.rst import directives
 
 from sphinx import addnodes
 from sphinx.directives import ObjectDescription
 from sphinx.domains import Domain, ObjType
 from sphinx.domains.python import _pseudo_parse_arglist
-from sphinx.locale import l_, _
+from sphinx.locale import _
 from sphinx.roles import XRefRole
 from sphinx.util.docfields import Field, GroupedField, TypedField
+from sphinx.util.docutils import SphinxDirective
 from sphinx.util.nodes import make_refnode
 
-if TYPE_CHECKING:
+if False:
+    # For type annotation
     from typing import Any, Dict, Iterator, List, Tuple  # NOQA
-    from docutils import nodes  # NOQA
     from sphinx.application import Sphinx  # NOQA
     from sphinx.builders import Builder  # NOQA
     from sphinx.environment import BuildEnvironment  # NOQA
@@ -40,14 +38,14 @@ class JSObject(ObjectDescription):
     has_arguments = False
 
     #: what is displayed right before the documentation entry
-    display_prefix = None  # type: unicode
+    display_prefix = None  # type: str
 
     #: If ``allow_nesting`` is ``True``, the object prefixes will be accumulated
     #: based on directive nesting
     allow_nesting = False
 
     def handle_signature(self, sig, signode):
-        # type: (unicode, addnodes.desc_signature) -> Tuple[unicode, unicode]
+        # type: (str, addnodes.desc_signature) -> Tuple[str, str]
         """Breaks down construct signatures
 
         Parses out prefix and argument list from construct definition. The
@@ -101,7 +99,7 @@ class JSObject(ObjectDescription):
         return fullname, prefix
 
     def add_target_and_index(self, name_obj, sig, signode):
-        # type: (Tuple[unicode, unicode], unicode, addnodes.desc_signature) -> None
+        # type: (Tuple[str, str], str, addnodes.desc_signature) -> None
         mod_name = self.env.ref_context.get('js:module')
         fullname = (mod_name and mod_name + '.' or '') + name_obj[0]
         if fullname not in self.state.document.ids:
@@ -125,7 +123,7 @@ class JSObject(ObjectDescription):
                                               '', None))
 
     def get_index_text(self, objectname, name_obj):
-        # type: (unicode, Tuple[unicode, unicode]) -> unicode
+        # type: (str, Tuple[str, str]) -> str
         name, obj = name_obj
         if self.objtype == 'function':
             if not obj:
@@ -202,15 +200,15 @@ class JSCallable(JSObject):
     has_arguments = True
 
     doc_field_types = [
-        TypedField('arguments', label=l_('Arguments'),
+        TypedField('arguments', label=_('Arguments'),
                    names=('argument', 'arg', 'parameter', 'param'),
                    typerolename='func', typenames=('paramtype', 'type')),
-        GroupedField('errors', label=l_('Throws'), rolename='err',
+        GroupedField('errors', label=_('Throws'), rolename='err',
                      names=('throws', ),
                      can_collapse=True),
-        Field('returnvalue', label=l_('Returns'), has_arg=False,
+        Field('returnvalue', label=_('Returns'), has_arg=False,
               names=('returns', 'return')),
-        Field('returntype', label=l_('Return type'), has_arg=False,
+        Field('returntype', label=_('Return type'), has_arg=False,
               names=('rtype',)),
     ]
 
@@ -221,7 +219,7 @@ class JSConstructor(JSCallable):
     allow_nesting = True
 
 
-class JSModule(Directive):
+class JSModule(SphinxDirective):
     """
     Directive to mark description of a new JavaScript module.
 
@@ -250,16 +248,15 @@ class JSModule(Directive):
 
     def run(self):
         # type: () -> List[nodes.Node]
-        env = self.state.document.settings.env
         mod_name = self.arguments[0].strip()
-        env.ref_context['js:module'] = mod_name
+        self.env.ref_context['js:module'] = mod_name
         noindex = 'noindex' in self.options
-        ret = []
+        ret = []  # type: List[nodes.Node]
         if not noindex:
-            env.domaindata['js']['modules'][mod_name] = env.docname
+            self.env.domaindata['js']['modules'][mod_name] = self.env.docname
             # Make a duplicate entry in 'objects' to facilitate searching for
             # the module in JavaScriptDomain.find_obj()
-            env.domaindata['js']['objects'][mod_name] = (env.docname, 'module')
+            self.env.domaindata['js']['objects'][mod_name] = (self.env.docname, 'module')
             targetnode = nodes.target('', '', ids=['module-' + mod_name],
                                       ismod=True)
             self.state.document.note_explicit_target(targetnode)
@@ -273,7 +270,7 @@ class JSModule(Directive):
 
 class JSXRefRole(XRefRole):
     def process_link(self, env, refnode, has_explicit_title, title, target):
-        # type: (BuildEnvironment, nodes.Node, bool, unicode, unicode) -> Tuple[unicode, unicode]  # NOQA
+        # type: (BuildEnvironment, nodes.Element, bool, str, str) -> Tuple[str, str]
         # basically what sphinx.domains.python.PyXRefRole does
         refnode['js:object'] = env.ref_context.get('js:object')
         refnode['js:module'] = env.ref_context.get('js:module')
@@ -297,12 +294,12 @@ class JavaScriptDomain(Domain):
     label = 'JavaScript'
     # if you add a new object type make sure to edit JSObject.get_index_string
     object_types = {
-        'function':  ObjType(l_('function'),  'func'),
-        'method':    ObjType(l_('method'),    'meth'),
-        'class':     ObjType(l_('class'),     'class'),
-        'data':      ObjType(l_('data'),      'data'),
-        'attribute': ObjType(l_('attribute'), 'attr'),
-        'module':    ObjType(l_('module'),    'mod'),
+        'function':  ObjType(_('function'),  'func'),
+        'method':    ObjType(_('method'),    'meth'),
+        'class':     ObjType(_('class'),     'class'),
+        'data':      ObjType(_('data'),      'data'),
+        'attribute': ObjType(_('attribute'), 'attr'),
+        'module':    ObjType(_('module'),    'mod'),
     }
     directives = {
         'function':  JSCallable,
@@ -323,10 +320,10 @@ class JavaScriptDomain(Domain):
     initial_data = {
         'objects': {},  # fullname -> docname, objtype
         'modules': {},  # mod_name -> docname
-    }  # type: Dict[unicode, Dict[unicode, Tuple[unicode, unicode]]]
+    }  # type: Dict[str, Dict[str, Tuple[str, str]]]
 
     def clear_doc(self, docname):
-        # type: (unicode) -> None
+        # type: (str) -> None
         for fullname, (pkg_docname, _l) in list(self.data['objects'].items()):
             if pkg_docname == docname:
                 del self.data['objects'][fullname]
@@ -335,7 +332,7 @@ class JavaScriptDomain(Domain):
                 del self.data['modules'][mod_name]
 
     def merge_domaindata(self, docnames, otherdata):
-        # type: (List[unicode], Dict) -> None
+        # type: (List[str], Dict) -> None
         # XXX check duplicates
         for fullname, (fn, objtype) in otherdata['objects'].items():
             if fn in docnames:
@@ -345,7 +342,7 @@ class JavaScriptDomain(Domain):
                 self.data['modules'][mod_name] = pkg_docname
 
     def find_obj(self, env, mod_name, prefix, name, typ, searchorder=0):
-        # type: (BuildEnvironment, unicode, unicode, unicode, unicode, int) -> Tuple[unicode, Tuple[unicode, unicode]]  # NOQA
+        # type: (BuildEnvironment, str, str, str, str, int) -> Tuple[str, Tuple[str, str]]
         if name[-2:] == '()':
             name = name[:-2]
         objects = self.data['objects']
@@ -371,7 +368,7 @@ class JavaScriptDomain(Domain):
 
     def resolve_xref(self, env, fromdocname, builder, typ, target, node,
                      contnode):
-        # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
+        # type: (BuildEnvironment, str, Builder, str, str, addnodes.pending_xref, nodes.Element) -> nodes.Element  # NOQA
         mod_name = node.get('js:module')
         prefix = node.get('js:object')
         searchorder = node.hasattr('refspecific') and 1 or 0
@@ -383,7 +380,7 @@ class JavaScriptDomain(Domain):
 
     def resolve_any_xref(self, env, fromdocname, builder, target, node,
                          contnode):
-        # type: (BuildEnvironment, unicode, Builder, unicode, nodes.Node, nodes.Node) -> List[Tuple[unicode, nodes.Node]]  # NOQA
+        # type: (BuildEnvironment, str, Builder, str, addnodes.pending_xref, nodes.Element) -> List[Tuple[str, nodes.Element]]  # NOQA
         mod_name = node.get('js:module')
         prefix = node.get('js:object')
         name, obj = self.find_obj(env, mod_name, prefix, target, None, 1)
@@ -394,13 +391,13 @@ class JavaScriptDomain(Domain):
                               name.replace('$', '_S_'), contnode, name))]
 
     def get_objects(self):
-        # type: () -> Iterator[Tuple[unicode, unicode, unicode, unicode, unicode, int]]
+        # type: () -> Iterator[Tuple[str, str, str, str, str, int]]
         for refname, (docname, type) in list(self.data['objects'].items()):
             yield refname, refname, type, docname, \
                 refname.replace('$', '_S_'), 1
 
     def get_full_qualified_name(self, node):
-        # type: (nodes.Node) -> unicode
+        # type: (nodes.Element) -> str
         modname = node.get('js:module')
         prefix = node.get('js:object')
         target = node.get('reftarget')
@@ -411,11 +408,12 @@ class JavaScriptDomain(Domain):
 
 
 def setup(app):
-    # type: (Sphinx) -> Dict[unicode, Any]
+    # type: (Sphinx) -> Dict[str, Any]
     app.add_domain(JavaScriptDomain)
 
     return {
         'version': 'builtin',
+        'env_version': 1,
         'parallel_read_safe': True,
         'parallel_write_safe': True,
     }
